<?php

namespace Sendcloud\Shipping\Models;

use DateTime;

class Nominated_Day_Meta {
	const DATE_FORMAT = 'Y-m-d\TH:i:s.uZ';
	/**
	 * @var DateTime
	 */
	private $delivery_date;

	/**
	 * @var string
	 */
	private $formatted_delivery_date;

	/**
	 * @var DateTime
	 */
	private $parcel_handover_date;

	/**
	 * @var string
	 */
	private $formatted_parcel_handover_date;

	/**
	 * Return object as array
	 *
	 * @return array
	 */
	public function to_array() {
		return array(
			'delivery_date'                  => $this->get_delivery_date(),
			'formatted_delivery_date'        => $this->get_formatted_delivery_date(),
			'parcel_handover_date'           => $this->get_parcel_handover_date(),
			'formatted_parcel_handover_date' => $this->get_formatted_parcel_handover_date(),
		);
	}

	/**
	 * Creates object from array
	 *
	 * @param $data
	 *
	 * @return Nominated_Day_Meta
	 */
	public static function from_array( $data ) {
		$instance = new self();

		$delivery_date                  = array_key_exists( 'delivery_date', $data ) ? $data['delivery_date'] : null;
		$formatted_delivery_date        = array_key_exists( 'formatted_delivery_date',
			$data ) ? $data['formatted_delivery_date'] : null;
		$parcel_handover_date           = array_key_exists( 'parcel_handover_date',
			$data ) ? $data['parcel_handover_date'] : null;
		$formatted_parcel_handover_date = array_key_exists( 'formatted_parcel_handover_date',
			$data ) ? $data['formatted_parcel_handover_date'] : null;

		$instance->set_delivery_date( DateTime::createFromFormat( self::DATE_FORMAT,
			$delivery_date ? $delivery_date : new DateTime() )->setTimezone( new \DateTimeZone( 'UTC' ) ) );

		$instance->set_formatted_delivery_date( $formatted_delivery_date ? $formatted_delivery_date : $instance->get_delivery_date()
		                                                                                                       ->format( self::DATE_FORMAT ) );

		$instance->set_parcel_handover_date( DateTime::createFromFormat( self::DATE_FORMAT,
			$parcel_handover_date ? $parcel_handover_date : new DateTime() )->setTimezone( new \DateTimeZone( 'UTC' ) ) );

		$instance->set_formatted_parcel_handover_date( $formatted_parcel_handover_date ? $formatted_parcel_handover_date : $instance->get_parcel_handover_date()
		                                                                                                                            ->format( self::DATE_FORMAT ) );

		return $instance;
	}

	/**
	 * @return \DateTime
	 */
	public function get_delivery_date() {
		return $this->delivery_date;
	}

	/**
	 * @param \DateTime $delivery_date
	 */
	public function set_delivery_date( $delivery_date ) {
		$this->delivery_date = $delivery_date;
	}

	/**
	 * @return string
	 */
	public function get_formatted_delivery_date() {
		return $this->formatted_delivery_date;
	}

	/**
	 * @param string $formatted_delivery_date
	 */
	public function set_formatted_delivery_date( $formatted_delivery_date ) {
		$this->formatted_delivery_date = $formatted_delivery_date;
	}

	/**
	 * @return \DateTime
	 */
	public function get_parcel_handover_date() {
		return $this->parcel_handover_date;
	}

	/**
	 * @param \DateTime $parcel_handover_date
	 */
	public function set_parcel_handover_date( $parcel_handover_date ) {
		$this->parcel_handover_date = $parcel_handover_date;
	}

	/**
	 * @return string
	 */
	public function get_formatted_parcel_handover_date() {
		return $this->formatted_parcel_handover_date;
	}

	/**
	 * @param string $formatted_parcel_handover_date
	 */
	public function set_formatted_parcel_handover_date( $formatted_parcel_handover_date ) {
		$this->formatted_parcel_handover_date = $formatted_parcel_handover_date;
	}
}