<?php

namespace SendCloud\Checkout\Validators;

use SendCloud\Checkout\Contracts\Validators\RequestValidator;
use SendCloud\Checkout\DTO\Validator;
use SendCloud\Checkout\Exceptions\ValidationException;
use SendCloud\Checkout\HTTP\Request;

/**
 * Class UpdateRequestValidator
 *
 * @package SendCloud\Checkout\Validators
 */
class UpdateRequestValidator implements RequestValidator
{
    /**
     * @var array[] Payload schema.
     */
    private static $schema = array(
        array(
            'field' => 'checkout_configuration',
            'type' => 'complex',
            'child' => array(
                array(
                    'field' => 'id',
                    'type' => 'string'
                ),
                array(
                    'field' => 'version',
                    'type' => 'int'
                ),
                array(
                    'field' => 'updated_at',
                    'type' => 'string'
                ),
                array(
                    'field' => 'delivery_zones',
                    'type' => 'collection',
                    'child' => array(
                        array(
                            'field' => 'id',
                            'type' => 'string'
                        ),
                        array(
                            'field' => 'location',
                            'type' => 'complex',
                            'child' => array(
                                array(
                                    'field' => 'country',
                                    'type' => 'complex',
                                    'child' => array(
                                        array(
                                            'field' => 'iso_2',
                                            'type' => 'string'
                                        ),
                                        array(
                                            'field' => 'name',
                                            'type' => 'string'
                                        ),
                                    )
                                )
                            )
                        ),
                        array(
                            'field' => 'delivery_methods',
                            'type' => 'collection',
                            'child' => array(
                                array(
                                    'field' => 'id',
                                    'type' => 'string'
                                ),
                                array(
                                    'field' => 'delivery_method_type',
                                    'type' => 'string'
                                ),
                                array(
                                    'field' => 'external_title',
                                    'type' => 'string'
                                ),
                                array(
                                    'field' => 'internal_title',
                                    'type' => 'string'
                                ),
                                array(
                                    'field' => 'carrier',
                                    'type' => 'complex',
                                    'child' => array(
                                        array(
                                            'field' => 'name',
                                            'type' => 'string'
                                        ),
                                        array(
                                            'field' => 'code',
                                            'type' => 'string'
                                        ),
                                        array(
                                            'field' => 'logo_url',
                                            'type' => 'string'
                                        ),
                                    )
                                ),
                                array(
                                    'field' => 'sender_address_id',
                                    'type' => 'int'
                                ),
                                array(
                                    'field' => 'show_carrier_information_in_checkout',
                                    'type' => 'bool'
                                ),
                                array(
                                    'field' => 'shipping_product',
                                    'type' => 'complex',
                                    'child' => array(
                                        array(
                                            'field' => 'code',
                                            'type' => 'string'
                                        ),
                                        array(
                                            'field' => 'name',
                                            'type' => 'string'
                                        ),
                                        array(
                                            'field' => 'lead_time_hours',
                                            'type' => 'int'
                                        ),
                                        array(
                                            'field' => 'selected_functionalities',
                                            'type' => 'array'
                                        ),
                                        array(
                                            'field' => 'carrier_delivery_days',
                                            'type' => 'collection',
                                            'contains_null' => true,
                                            'child' => array(
                                                array(
                                                    'field' => 'enabled',
                                                    'type' => 'bool'
                                                ), array(
                                                    'field' => 'start_time_hours',
                                                    'type' => 'int',
                                                    'nullable' => true,
                                                ), array(
                                                    'field' => 'start_time_minutes',
                                                    'type' => 'int',
                                                    'nullable' => true,
                                                ), array(
                                                    'field' => 'end_time_hours',
                                                    'type' => 'int',
                                                    'nullable' => true,
                                                ), array(
                                                    'field' => 'end_time_minutes',
                                                    'type' => 'int',
                                                    'nullable' => true,
                                                ),
                                            )
                                        ),
                                    )
                                ),
                                array(
                                    'field' => 'parcel_handover_days',
                                    'type' => 'collection',
                                    'contains_null' => true,
                                    'child' => array(
                                        array(
                                            'field' => 'enabled',
                                            'type' => 'bool'
                                        ),
                                        array(
                                            'field' => 'cut_off_time_hours',
                                            'type' => 'int'
                                        ),
                                        array(
                                            'field' => 'cut_off_time_minutes',
                                            'type' => 'int'
                                        ),
                                    )
                                ),
                            )
                        ),
                    )
                ),
            )
        )
    );

    /**
     * Validates array.
     *
     * @param Request $request
     *
     * @return void
     *
     * @throws ValidationException
     */
    public function validate(Request $request)
    {
        $body = $request->getBody();
        if (empty($body)) {
            throw new ValidationException(array(array('location' => 'body', 'message' => 'Request payload is empty.')));
        }

        Validator::validate(static::$schema, $request->getBody());
    }
}