<?php

namespace SendCloud\Tests\Checkout\Domain\DeliveryMethod;

use SendCloud\Checkout\API\Checkout\Checkout;
use SendCloud\Checkout\Domain\Delivery\DeliveryMethod;
use SendCloud\Tests\Common\BaseTest;

class DeliveryMethodTest extends BaseTest
{
    /**
     * @var Checkout
     */
    public $checkout;
    /**
     * @var \SendCloud\Checkout\API\Checkout\Delivery\Method\DeliveryMethod[]
     */
    public $deliveryMethods;

    public function setUp()
    {
        $payload = json_decode(file_get_contents(__DIR__ . '/../../../Common/APIDataExamples/checkout.json'), true);
        $this->checkout = Checkout::fromArray($payload['checkout_configuration']);
        $deliveryZones = $this->checkout->getDeliveryZones();
        $deliveryZone = $deliveryZones[0];
        $this->deliveryMethods = $deliveryZone->getDeliveryMethods();
    }

    public function testInstantiationFromDto()
    {
        // arrange
        $dto = $this->deliveryMethods[0];

        // act
        $method = DeliveryMethod::fromDTO($dto);

        // assert
        $this->assertInstanceOf('SendCloud\Checkout\Domain\Delivery\DeliveryMethod', $method);
        $this->assertEquals($method->getType(), $dto->getType());
        $this->assertEquals($method->getInternalTitle(), $dto->getInternalTitle());
        $this->assertEquals($method->getExternalTitle(), $dto->getExternalTitle());
        $this->assertEquals($method->getRawConfig(), json_encode($dto->toArray()));
        $carrier = $method->getCarrier();
        $this->assertEquals($carrier->getName(), $dto->getCarrier()->getName());
        $this->assertEquals($carrier->getCode(), $dto->getCarrier()->getCode());
        $this->assertEquals($carrier->getLogoUrl(), $dto->getCarrier()->getLogoUrl());
        $shippingProduct = $method->getShippingProduct();
        $this->assertEquals($shippingProduct->getCode(), $dto->getShippingProduct()->getCode());
        $this->assertEquals($shippingProduct->getName(), $dto->getShippingProduct()->getName());
        $this->assertEquals($shippingProduct->getLeadTimeHours(), $dto->getShippingProduct()->getLeadTimeHours());
        $this->assertEquals($shippingProduct->getSelectedFunctionalities(), $dto->getShippingProduct()->getSelectedFunctionalities());
        $targetDeliveryDays = $dto->getShippingProduct()->getCarrierDeliveryDays();
        foreach ($shippingProduct->getDeliveryDays() as $index => $deliveryDay) {
            $targetDay = $targetDeliveryDays[$index];
            if ($deliveryDay === null) {
                $this->assertNull($targetDay);

                continue;
            }

            $this->assertEquals($deliveryDay->getEndingMinute(), $targetDay->getEndingMinute());
            $this->assertEquals($deliveryDay->getEndingHour(), $targetDay->getEndingHour());
            $this->assertEquals($deliveryDay->getStartingMinute(), $targetDay->getStartingMinute());
            $this->assertEquals($deliveryDay->getStartingHour(), $targetDay->getStartingHour());
        }

        $handoverDays = $dto->getHandoverDays();
        foreach ($method->getHandoverDays() as $index => $handoverDay) {
            $targetHandoverDay = $handoverDays[$index];
            if ($handoverDay === null) {
                $this->assertNull($targetHandoverDay);

                continue;
            }

            $this->assertEquals($handoverDay->isEnabled(), $targetHandoverDay->isEnabled());
            $this->assertEquals($handoverDay->getCutOffMinute(), $targetHandoverDay->getCutOffTimeMinutes());
            $this->assertEquals($handoverDay->getCutOffHour(), $targetHandoverDay->getCutOffTimeHours());
        }
    }

    public function testCanBeUpdatedNothingChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertFalse($result);
    }

    public function testCanBeUpdatedIdChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setId('123123123');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertFalse($result);
    }

    public function testCanBeUpdatedSystemIdChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setSystemId(123123123);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertFalse($result);
    }

    public function testCanBeUpdatedDeliveryZoneIdChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setDeliveryZoneId('123123123');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedCarrierCodeChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getCarrier()->setCode('123123');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedCarrierNameChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getCarrier()->setName('name changed');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedInternalTitleChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setInternalTitle('title changed');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedExternalTitleChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setExternalTitle('title changed');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedSenderAddressIdChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setSenderAddressId(234234);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }


    public function testCanBeUpdatedDisplayCarrierInfoChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setShowCarrierInfoOnCheckout(!$method1->isShowCarrierInfoOnCheckout());

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedSelectedFunctionalitiesChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getShippingProduct()->setSelectedFunctionalities(array('signature' => false));

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedCarrierLogoUrlChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getCarrier()->setLogoUrl('name changed');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductNameChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getShippingProduct()->setName('new name');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductCodeChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getShippingProduct()->setCode('new test');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductDeliveryEnabledChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $deliveryDays = $method1->getShippingProduct()->getDeliveryDays();
        $day = $deliveryDays['friday'];
        $day->setEnabled(false);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductDeliveryDateEndingMinuteChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $deliveryDays = $method1->getShippingProduct()->getDeliveryDays();
        $day = $deliveryDays['friday'];
        $day->setEndingMinute(23);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductDeliveryDateEndingHourChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $deliveryDays = $method1->getShippingProduct()->getDeliveryDays();
        $day = $deliveryDays['monday'];
        $day->setEndingHour(23);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductDeliveryDateStartingHourChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $deliveryDays = $method1->getShippingProduct()->getDeliveryDays();
        $day = $deliveryDays['saturday'];
        $day->setStartingHour(11);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedShippingProductDeliveryDateStartingMinuteChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $deliveryDays = $method1->getShippingProduct()->getDeliveryDays();
        $day = $deliveryDays['thursday'];
        $day->setStartingMinute(21);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedLeadHourChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->getShippingProduct()->setLeadTimeHours(23);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedTypeChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $method1->setType('75');

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedHandoverDayEnabledChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $handoverDays = $method1->getHandoverDays();
        $day = $handoverDays['monday'];
        $day->setEnabled(false);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedHandoverCutOffMinuteChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $handoverDays = $method1->getHandoverDays();
        $day = $handoverDays['tuesday'];
        $day->setCutOffMinute(12);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }

    public function testCanBeUpdatedHandoverDayCutOffHourChanged()
    {
        // arrange
        $dto = $this->deliveryMethods[0];
        $method1 = DeliveryMethod::fromDTO($dto);
        $method2 = DeliveryMethod::fromDTO($dto);
        $handoverDays = $method1->getHandoverDays();
        $day = $handoverDays['sunday'];
        $day->setCutOffHour(21);

        // act
        $result = $method1->canBeUpdated($method2);

        // assert
        $this->assertTrue($result);
    }
}