<?php

namespace SendCloud\Tests\Checkout\Validators;

use SendCloud\Checkout\Exceptions\ValidationException;
use SendCloud\Checkout\HTTP\Request;
use SendCloud\Checkout\Validators\UpdateRequestValidator;
use SendCloud\Tests\Common\BaseTest;

/**
 * Class UpdateRequestValidatorTest
 *
 * @package SendCloud\Tests\Checkout\Validators
 */
class UpdateRequestValidatorTest extends BaseTest
{
    /**
     * @var UpdateRequestValidator
     */
    public $validator;
    /**
     * @var mixed
     */
    public $payload;

    public function setUp()
    {
        $this->validator = new UpdateRequestValidator();
        $this->payload = json_decode(file_get_contents(__DIR__ . '/../../Common/APIDataExamples/invalidCheckout.json'), true);
    }

    /**
     * @expectedException \SendCloud\Checkout\Exceptions\ValidationException
     */
    public function testEmptyBody()
    {
        // arrange
        $request = new Request(array(), array());

        // act
        $this->validator->validate($request);
    }

    /**
     * @expectedException \SendCloud\Checkout\Exceptions\ValidationException
     */
    public function testInvalidPayload()
    {
        // arrange
        $request = new Request($this->payload, array());

        // act
        $this->validator->validate($request);
    }

    public function testInvalidPayloadErrors()
    {
        // Arrange.
        $request = new Request($this->payload, array());
        $expected = array (
            0 =>
                array (
                    'location' => 'checkout_configuration.id',
                    'message' => 'Field is required.',
                ),
            1 =>
                array (
                    'location' => 'checkout_configuration.version',
                    'message' => 'Field cannot be null.',
                ),
            2 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[0].id',
                    'message' => 'Field is required.',
                ),
            3 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[0].location.country.iso_2',
                    'message' => 'Field is required.',
                ),
            4 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[0].location.country.name',
                    'message' => 'Field cannot be null.',
                ),
            5 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[0].delivery_methods',
                    'message' => 'Collection cannot contain null values.',
                ),
            6 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[1].delivery_methods[0].id',
                    'message' => 'Field cannot be null.',
                ),
            7 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[1].delivery_methods[0].delivery_method_type',
                    'message' => 'Field cannot be null.',
                ),
            8 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[1].delivery_methods[0].external_title',
                    'message' => 'Field is required.',
                ),
            9 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[1].delivery_methods[0].internal_title',
                    'message' => 'Field is required.',
                ),
            10 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[2].delivery_methods[0].carrier.name',
                    'message' => 'Field is required.',
                ),
            11 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[2].delivery_methods[0].carrier.code',
                    'message' => 'Field cannot be null.',
                ),
            12 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[2].delivery_methods[0].carrier.logo_url',
                    'message' => 'Field is required.',
                ),
            13 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[2].delivery_methods[0].sender_address_id',
                    'message' => 'Field is required.',
                ),
            14 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[2].delivery_methods[0].shipping_product',
                    'message' => 'Field cannot be null.',
                ),
            15 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[3].delivery_methods[0].parcel_handover_days[friday].enabled',
                    'message' => 'Field cannot be null.',
                ),
            16 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[3].delivery_methods[0].parcel_handover_days[friday].cut_off_time_hours',
                    'message' => 'Field is required.',
                ),
            17 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[3].delivery_methods[0].parcel_handover_days[monday].cut_off_time_minutes',
                    'message' => 'Field is required.',
                ),
            18 =>
                array (
                    'location' => 'checkout_configuration.delivery_zones[4].delivery_methods[0].show_carrier_information_in_checkout',
                    'message' => 'Field is required.',
                ),
        );

        $actual = array();

        // act
        try {
            $this->validator->validate($request);
        } catch (ValidationException $e) {
            $actual = $e->getValidationErrors();
        }

        // assert
        $this->assertEquals($expected, $actual);
    }
}